function ReportDecorator(eventAggregator, restEngine, templateManager, ZhLogger, ScreenService, eventEngine) {
    
    this.onContainerOpen = function() {
        try {
            var container = templateManager.container;
            validContainer(container);
            if (container.customReportsConfig) {
                var customReportsConfig = container.customReportsConfig;
                validCustomReportsConfig(customReportsConfig);
                var widget = selectWidget(container, customReportsConfig.widgetName);
                var containerName = container.customReportsConfig ? container.customReportsConfig.reportCodeName : container.name;
                
                var fieldToChangeData = selectField(widget, customReportsConfig.fieldName);
                
                if (customReportsConfig.reportIsTheMainFunction === undefined || customReportsConfig.reportIsTheMainFunction === true) {
                    buildData(containerName, fieldToChangeData);
                } else {
                    const separator = "', '";
                    const eventCode = "ReportDecorator.triggerBuildData('" + containerName + separator + customReportsConfig.widgetName + separator + customReportsConfig.fieldName + "')";
                    
                    addEvent({
                        entity: widget,
                        event: {
                            name: 'WidgetOnEnterOnce',
                            code: eventCode,
                            bind: true,
                        }
                    });
                }
                widget.beforeOpenReport = beforeOpenReport;
            }
        } catch (e) {
            ZhLogger.warning(e);
        }
    };
    eventAggregator.subscribe('onContainerOpen',this.onContainerOpen);
    
    this.triggerBuildData = function(containerName, widgetName, fieldName) {
        const container = templateManager.container;
        
        validContainer(container);
        
        const widget = selectWidget(container, widgetName);
        const fieldToChangeData = selectField(widget, fieldName);
        
        try {
            buildData(containerName, fieldToChangeData);
        } catch (e) {
            ZhLogger.warning(e);
        }
    };
    
    function buildData(containerName, fieldToChangeData) {
        const route = getServiceUrlBasedOnTriggerPackage() + '#/lib_getDataReportField';
        
        restEngine.post(route, {
            "requestType": "Row",
            "row": [containerName.substr(0, 6)]
        }).then(function (callback) {
            try {
                var data = callback.dataset.dataReport;
                
                handleErrors(data);
                
                const formattedData = getFormattedData(fieldToChangeData, data);
                
                formattedData.forEach(function(row) {
                    row.description = ScreenService.i18n(row.description);
                });
                
                setDataSource(fieldToChangeData, formattedData);
                applyDefaultValue(fieldToChangeData);
                
                /**
                 * Força o select do item para que haja alteração visual.
                 * Operações visuais como updateTemplate e redraw não surtiram efeitos.
                 */
                setTimeout(function() {
                    clickOnSelectedOption(fieldToChangeData.name);
                }, 500);
                
                addEvent({
                    entity: fieldToChangeData.getParent(),
                    event: {
                        name: 'WidgetOnEnter',
                        code: "ReportDecorator.onEnterReportWidget('" + fieldToChangeData.name + "')",
                        bind: true,
                    }
                });
            } catch (e) {
                ZhLogger.warning(e);
            }
        }.bind(this), function(error) {
            if (error.data && error.data.error) {
                throw error.data.error;
            } else {
                throw ScreenService.i18n('Ocorreu um erro ao buscar a lista de relátorios da tela.');
            }
        });
    }
    
    function getServiceUrlBasedOnTriggerPackage() {
        const fileName = templateManager.container.fileName;
        const aliasSplited = fileName.split('#').shift().split('_');
        
        if (aliasSplited.length) {
            const packageName = aliasSplited.shift();
            
            if (packageName) return packageName;
        }
        
        return 'lib';
    }
    
    function getFormattedData(fieldToChangeData, data) {
        return data
                .filter(function(reportElement) {
                    return !Array.isArray(reportElement);
                })
                .map(function(reportElement) {
                    if (Array.isArray(reportElement)) return;
                    
                    const newElement = {};
                    
                    newElement[fieldToChangeData.valueField] = reportElement.value;
                    newElement[fieldToChangeData.descriptionField] = reportElement.description;
                    newElement[fieldToChangeData.strategyField] = reportElement.strategy;
                    
                    return newElement;
                });
    }
    
    function handleErrors(data) {
        const errorIndex = data.findIndex(function(reportElement) {
            return Array.isArray(reportElement);
        });
        
        if (errorIndex === -1) return;
            
        if (data.length === 1) {
            const message = ScreenService.i18n('Ocorreu um erro ao buscar a lista de relatórios da tela.');
            
            ScreenService.errorNotification(message);
            
            throw message + ScreenService.i18n('Consulte a requisição correspondente para mais detalhes.');
        } else {
            const isQr2Parametrized = data[errorIndex].findIndex(function(error) {
                return error === ScreenService.i18n('Servidor de relatórios QR2 não está parametrizado.');
            }) === -1;
            
            const isBirtParametrized = data[errorIndex].findIndex(function(error) {
                return error === ScreenService.i18n('Servidor de relatórios BIRT não está parametrizado.');
            }) === -1;
            
            if (isQr2Parametrized || isBirtParametrized) {
                ScreenService.errorNotification(ScreenService.i18n('Alguns relatórios podem não estar disponíveis.'));
            }
            
        	ZhLogger.warning(ScreenService.i18n('Alguns relatórios podem não estar disponíveis. Consulte a requisição correspondente para mais detalhes.'));
        }
    }
    
    function setDataSource(fieldToChangeData, data) {
        if (!fieldToChangeData.dataSource.data || fieldToChangeData.dataSource.data.length == 0) {
            fieldToChangeData.dataSource.data = data;
        } else {
            const valueField = fieldToChangeData.valueField;
            const descriptionField = fieldToChangeData.descriptionField;
            const strategyField = fieldToChangeData.strategyField;
            
            const removingDuplicatedReports = function(report, index, reports) {
                return index === reports.findIndex(function(r) {
                    return r[valueField] === report[valueField] &&
                           r[descriptionField] === report[descriptionField] &&
                           r[strategyField] === report[strategyField];
                });
            };
            
            const newData = fieldToChangeData.dataSource.data.concat(data);
            
            fieldToChangeData.dataSource.data = newData.filter(removingDuplicatedReports);
        }
    }
    
    function applyDefaultValue(fieldToChangeData) {
        if (!fieldToChangeData.defaultValue || fieldToChangeData.defaultValue === 'firstRow') {
            const firstRow = fieldToChangeData.dataSource.getFirstRow();
            
            fieldToChangeData.defaultValue = firstRow[fieldToChangeData.valueField];
        }
        
        fieldToChangeData.applyDefaultValue();
    }
    
    function clickOnSelectedOption(fieldName) {
        const baseSelector = "#" + fieldName + " + div";
        const baseElement = document.querySelector(baseSelector);
        
        if (!baseElement) {
            return setTimeout(function recursiveCallClickOnSelectedOption() {
                clickOnSelectedOption(fieldName);
            }, 500);
        }
        
        document.querySelector(baseSelector).click();
        
        const selectedElementSelector = baseSelector + ' li[class*="selected"]';
        
        document.querySelector(selectedElementSelector).click();
    }
    
    function selectWidget(container, widgetName) {
        var widget = container.getWidget(widgetName);
        
        if (widget) {
            return widget;
        } else {
            throw ScreenService.i18n("A widget informada não existe dentro desse container.");
        }
    }
    
    function selectField(widget, fieldName) {
        var field = widget.getField(fieldName);
        
        if (field) {
            if (field.dataSource) {
                return field;
            } else {
                throw ScreenService.i18n("O field informado não contém a propriedade dataSource.");    
            }
        } else {
            throw ScreenService.i18n("O field informado não existe na widget informada.");
        }
    }
    
    function validContainer(container) {
        if (!container) {
            throw ScreenService.i18n("Container não existe.");
        }
    }
    
    function validCustomReportsConfig(config) {
        if (!config.widgetName) {
            throw ScreenService.i18n('A propriedade widgetName não existe dentro de customReportsConfig.');
        }
        
        if (!config.fieldName) {
            throw ScreenService.i18n('A propriedade fieldName não existe dentro de customReportsConfig.');
        }
    }
    
    function eventExistsByCode(events, searchedCode) {
        return events.filter(function(event){
            return event.code.indexOf(searchedCode) != -1;
        }).length > 0;
    }
    
    function eventExistsByName(events, searchedName) {
        return events.findIndex(function(event){
            return event.name === searchedName;
        }) !== -1;
    }
    
    function addCodeOnEvent(events, eventName, eventCode, placeOn) {
        if (!placeOn) placeOn = 'right';
        
        const eventIndex = events.findIndex(function(event) {
            return event.name === eventName;
        });
        const event = events[eventIndex];
        const endSemicolonIndex = event.code.lastIndexOf(';');
        const codeHasEndSemicolon = endSemicolonIndex !== -1 && endSemicolonIndex === event.code.length - 1;
        
        if (!codeHasEndSemicolon) {
            event.code += ';';
        }
        
        if (placeOn === 'right') {
            events[eventIndex].code += eventCode;
        } else {
            const newCode = eventCode + ';' + events[eventIndex].code;
            
            events[eventIndex].code = newCode;
        }
    }
    
    function addEvent(params) {
        const entity = params.entity;
        const event = params.event;
        
        const name = event.name;
        const code = event.code;
        const events = entity.events;
        
        if (eventExistsByName(events, name)) {
            if (!eventExistsByCode(events, code)) {
                addCodeOnEvent(events, name, code, event.placeOn);
            }
        } else {
            events.push({
                name: name,
                code: code,
            });
            
            if (event.bind) {
                eventEngine.bindEvent(entity, name, code);
            }
        }
    }
    
    function getSelectedOption(fieldName) {
        const selector = '#' + fieldName + ' + div li[class*="selected"]';
        
        return document.querySelector(selector);
    }
    
    function getCurrent(fieldName) {
        const selector = '#' + fieldName + ' + div > span';
        
        return document.querySelector(selector).textContent;
    }
    
    this.onEnterReportWidget = function(fieldName) {
        const field = templateManager.container.getActiveWidget().getField(fieldName);
        
        if (!field.value()) {
            applyDefaultValue(field);
        }
        
        if (!getSelectedOption(fieldName)) {
            setTimeout(function runClickOnSelectedOption() {
                const selectedOption = getSelectedOption(fieldName);
                
                if (!selectedOption) {
                    return setTimeout(runClickOnSelectedOption, 500);
                }
                
                const current = getCurrent(fieldName);
                
                if (!current) {
                    return clickOnSelectedOption(fieldName);
                }
            }, 500);
        }
    };
    
    function beforeOpenReport(params) {
        const reportParams = params.reportParams;
        const reportNameSplitted = reportParams.__report_name.split('.');
        const reportName = reportNameSplitted[0];
        const strategy = reportNameSplitted[1];
        
        reportParams.__report_name = reportName;
        reportParams.STRATEGY = strategy;
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register('ReportDecorator', ReportDecorator);
});